#include <iostream>
#include <vector>
#include <sstream>
#include <string>
#include "histogram.h"
#include "svg.h"
#include "text.h"
#include <curl/curl.h>

using namespace std;

struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};


Input input_data(istream& in, bool prompt = true) {  //   prompt    
    Input data;
    size_t number_count;

    if (prompt) {
        cerr << "Enter number count: ";
    }
    in >> number_count;

    data.numbers.resize(number_count);
    if (prompt) {
        cerr << "Enter numbers: ";
    }
    for (size_t i = 0; i < number_count; i++) {
        in >> data.numbers[i];
    }

    if (prompt) {
        cerr << "Enter number of bins: ";
    }
    in >> data.bin_count;

    return data;
}

// Callback     stringstream

static size_t Write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    auto* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}


Input download(const string& address) {
    stringstream buffer;
    CURL* curl = curl_easy_init();

    if (!curl) {
        cerr << ":    cURL" << endl;
        exit(1);
    }

    //  cURL
    curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, Write_data);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L); //   SSL  
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);

    //  
    CURLcode res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        cerr << " : " << curl_easy_strerror(res) << endl;
        exit(1);
    }

    curl_easy_cleanup(curl);
    return input_data(buffer, false);
}

int main(int argc, char* argv[]) {
    curl_global_init(CURL_GLOBAL_ALL);

    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    }
    else {
        input = input_data(cin, true);
    }
    size_t max_count;
    auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    curl_global_cleanup();
    return 0;

}
